@@ -1,10 +1,9 @@
 #!/usr/bin/env python
 # -*- encoding: utf-8 -*-
 import logging
-import re
+from urllib.parse import unquote
 
 import requests
-import six
 from caldav.elements import cdav
 from caldav.elements import dav
 from caldav.elements import ical
@@ -19,13 +18,9 @@ from caldav.objects import log
 from caldav.objects import Principal
 from caldav.objects import ScheduleInbox
 from caldav.objects import ScheduleOutbox
+from caldav.requests import HTTPBearerAuth
 from lxml import etree
 
-if six.PY3:
-    from urllib.parse import unquote
-else:
-    from urlparse import unquote
-
 
 class DAVResponse:
     """
@@ -40,13 +35,18 @@ class DAVResponse:
     tree = None
     headers = {}
     status = 0
+    davclient = None
+    huge_tree = False
 
-    def __init__(self, response):
+    def __init__(self, response, davclient=None):
         self.headers = response.headers
         log.debug("response headers: " + str(self.headers))
         log.debug("response status: " + str(self.status))
 
         self._raw = response.content
+        self.davclient = davclient
+        if davclient:
+            self.huge_tree = davclient.huge_tree
 
         ## TODO: this if/else/elif could possibly be refactored, or we should
         ## consider to do streaming into the xmltree library as originally
@@ -69,12 +69,15 @@ class DAVResponse:
                 # self.tree = etree.parse(response.raw, parser=etree.XMLParser(remove_blank_text=True))
                 try:
                     self.tree = etree.XML(
-                        self._raw, parser=etree.XMLParser(remove_blank_text=True)
+                        self._raw,
+                        parser=etree.XMLParser(
+                            remove_blank_text=True, huge_tree=self.huge_tree
+                        ),
                     )
                 except:
                     logging.critical(
                         "Expected some valid XML from the server, but got this: \n"
-                        + self._raw,
+                        + str(self._raw),
                         exc_info=True,
                     )
                     raise
@@ -96,7 +99,10 @@ class DAVResponse:
             ## data be parsed through this code.
             try:
                 self.tree = etree.XML(
-                    self._raw, parser=etree.XMLParser(remove_blank_text=True)
+                    self._raw,
+                    parser=etree.XMLParser(
+                        remove_blank_text=True, huge_tree=self.huge_tree
+                    ),
                 )
             except:
                 pass
@@ -314,6 +320,7 @@ class DAVClient:
 
     proxy = None
     url = None
+    huge_tree = False
 
     def __init__(
         self,
@@ -325,6 +332,8 @@ class DAVClient:
         timeout=None,
         ssl_verify_cert=True,
         ssl_cert=None,
+        headers={},
+        huge_tree=False,
     ):
         """
         Sets up a HTTPConnection object towards the server in the url.
@@ -333,7 +342,9 @@ class DAVClient:
          * proxy: A string defining a proxy server: `hostname:port`
          * username and password should be passed as arguments or in the URL
          * auth, timeout and ssl_verify_cert are passed to requests.request.
-         ** ssl_verify_cert can be the path of a CA-bundle or False.
+         * ssl_verify_cert can be the path of a CA-bundle or False.
+         * huge_tree: boolean, enable XMLParser huge_tree to handle big events, beware
+           of security issues, see : https://lxml.de/api/lxml.etree.XMLParser-class.html
 
         The requests library will honor a .netrc-file, if such a file exists
         username and password may be omitted.  Known bug: .netrc is honored
@@ -344,12 +355,12 @@ class DAVClient:
 
         log.debug("url: " + str(url))
         self.url = URL.objectify(url)
-
+        self.huge_tree = huge_tree
         # Prepare proxy info
         if proxy is not None:
             self.proxy = proxy
             # requests library expects the proxy url to have a scheme
-            if re.match("^.*://", proxy) is None:
+            if "://" not in proxy:
                 self.proxy = self.url.scheme + "://" + proxy
 
             # add a port is one is not specified
@@ -361,11 +372,14 @@ class DAVClient:
             log.debug("init - proxy: %s" % (self.proxy))
 
         # Build global headers
-        self.headers = {
-            "User-Agent": "Mozilla/5.0",
-            "Content-Type": "text/xml",
-            "Accept": "text/xml, text/calendar",
-        }
+        self.headers = headers
+        self.headers.update(
+            {
+                "User-Agent": "Mozilla/5.0",
+                "Content-Type": "text/xml",
+                "Accept": "text/xml, text/calendar",
+            }
+        )
         if self.url.username is not None:
             username = unquote(self.url.username)
             password = unquote(self.url.password)
@@ -414,7 +428,7 @@ class DAVClient:
     def calendar(self, **kwargs):
         """Returns a calendar object.
 
-        Typically, an URL should be given as a named parameter (url)
+        Typically, a URL should be given as a named parameter (url)
 
         No network traffic will be initiated by this method.
 
@@ -552,12 +566,12 @@ class DAVClient:
     def extract_auth_types(self, header):
         auth_types = header.lower().split(",")
         auth_types = map(lambda auth_type: auth_type.strip(), auth_types)
-        auth_types = map(lambda auth_type: auth_type[: auth_type.find(" ")], auth_types)
+        auth_types = map(lambda auth_type: auth_type.split(" ")[0], auth_types)
         return list(filter(lambda auth_type: auth_type, auth_types))
 
     def request(self, url, method="GET", body="", headers={}):
         """
-        Actually sends the request
+        Actually sends the request, and does the authentication
         """
         combined_headers = self.headers.copy()
         combined_headers.update(headers)
@@ -591,7 +605,7 @@ class DAVClient:
                 cert=self.ssl_cert,
             )
             log.debug("server responded with %i %s" % (r.status_code, r.reason))
-            response = DAVResponse(r)
+            response = DAVResponse(r, self)
         except:
             ## this is a workaround needed due to some weird server
             ## that would just abort the connection rather than send a
@@ -614,20 +628,21 @@ class DAVClient:
         if (
             r.status_code == 401
             and "WWW-Authenticate" in r.headers
-            and self.password
-            and self.username
             and not self.auth
+            and self.username
         ):
             auth_types = self.extract_auth_types(r.headers["WWW-Authenticate"])
 
-            if "basic" in auth_types:
-                self.auth = requests.auth.HTTPBasicAuth(self.username, self.password)
-            elif "digest" in auth_types:
+            if self.password and self.username and "digest" in auth_types:
                 self.auth = requests.auth.HTTPDigestAuth(self.username, self.password)
+            elif self.password and self.username and "basic" in auth_types:
+                self.auth = requests.auth.HTTPBasicAuth(self.username, self.password)
+            elif self.password and "bearer" in auth_types:
+                self.auth = HTTPBearerAuth(self.password)
             else:
                 raise NotImplementedError(
                     "The server does not provide any of the currently "
-                    "supported authentication methods: basic, digest"
+                    "supported authentication methods: basic, digest, bearer"
                 )
 
             return self.request(url, method, body, headers)
@@ -635,27 +650,31 @@ class DAVClient:
         elif (
             r.status_code == 401
             and "WWW-Authenticate" in r.headers
-            and self.password
-            and self.username
             and self.auth
+            and self.password
+            and hasattr(self.password, "decode")
         ):
-
-            ## Some (ancient) servers don't like UTF-8 binary auth with Digest authentication.
-            ## An example are old SabreDAV based servers.  Not sure about UTF-8 and Basic Auth,
-            ## but likely the same.  so retry if password is a bytes sequence and not a string
-            ## (see commit 13a4714, which introduced this regression)
+            ## Most likely we're here due to wrong username/password
+            ## combo, but it could also be charset problems.  Some
+            ## (ancient) servers don't like UTF-8 binary auth with
+            ## Digest authentication.  An example are old SabreDAV
+            ## based servers.  Not sure about UTF-8 and Basic Auth,
+            ## but likely the same.  so retry if password is a bytes
+            ## sequence and not a string (see commit 13a4714, which
+            ## introduced this regression)
 
             auth_types = self.extract_auth_types(r.headers["WWW-Authenticate"])
 
-            if hasattr(self.password, "decode"):
-                if "basic" in auth_types:
-                    self.auth = requests.auth.HTTPBasicAuth(
-                        self.username, self.password.decode()
-                    )
-                elif "digest" in auth_types:
-                    self.auth = requests.auth.HTTPDigestAuth(
-                        self.username, self.password.decode()
-                    )
+            if "digest" in auth_types:
+                self.auth = requests.auth.HTTPDigestAuth(
+                    self.username, self.password.decode()
+                )
+            elif "basic" in auth_types:
+                self.auth = requests.auth.HTTPBasicAuth(
+                    self.username, self.password.decode()
+                )
+            elif "bearer" in auth_types:
+                self.auth = HTTPBearerAuth(self.password.decode())
 
             self.username = None
             self.password = None
