@@ -1,11 +1,12 @@
 #!/usr/bin/env python
 # -*- encoding: utf-8 -*-
-
 from caldav.lib.namespace import ns
-from .base import BaseElement, ValuedBaseElement
+
+from .base import BaseElement
+from .base import ValuedBaseElement
 
 
-## Operations
+# Operations
 class Propfind(BaseElement):
     tag = ns("D", "propfind")
 
@@ -17,11 +18,23 @@ class PropertyUpdate(BaseElement):
 class Mkcol(BaseElement):
     tag = ns("D", "mkcol")
 
-## Filters
 
-## Conditions
+class SyncCollection(BaseElement):
+    tag = ns("D", "sync-collection")
 
-## Components / Data
+
+# Filters
+
+# Conditions
+class SyncToken(BaseElement):
+    tag = ns("D", "sync-token")
+
+
+class SyncLevel(BaseElement):
+    tag = ns("D", "sync-level")
+
+
+# Components / Data
 
 
 class Prop(BaseElement):
@@ -36,7 +49,7 @@ class Set(BaseElement):
     tag = ns("D", "set")
 
 
-## Properties
+# Properties
 class ResourceType(BaseElement):
     tag = ns("D", "resourcetype")
 
@@ -45,10 +58,18 @@ class DisplayName(ValuedBaseElement):
     tag = ns("D", "displayname")
 
 
+class GetEtag(ValuedBaseElement):
+    tag = ns("D", "getetag")
+
+
 class Href(BaseElement):
     tag = ns("D", "href")
 
 
+class SupportedReportSet(BaseElement):
+    tag = ns("D", "supported-report-set")
+
+
 class Response(BaseElement):
     tag = ns("D", "response")
 
@@ -56,6 +77,22 @@ class Response(BaseElement):
 class Status(BaseElement):
     tag = ns("D", "status")
 
+
+class PropStat(BaseElement):
+    tag = ns("D", "propstat")
+
+
+class MultiStatus(BaseElement):
+    tag = ns("D", "multistatus")
+
+
 class CurrentUserPrincipal(BaseElement):
     tag = ns("D", "current-user-principal")
 
+
+class PrincipalCollectionSet(BaseElement):
+    tag = ns("D", "principal-collection-set")
+
+
+class Allprop(BaseElement):
+    tag = ns("D", "allprop")
