@@ -1,30 +1,14 @@
 #!/usr/bin/env python
 # -*- encoding: utf-8 -*-
+from urllib.parse import ParseResult
+from urllib.parse import quote
+from urllib.parse import SplitResult
+from urllib.parse import unquote
+from urllib.parse import urlparse
+from urllib.parse import urlunparse
+
 from caldav.lib.python_utilities import to_normal_str
 from caldav.lib.python_utilities import to_unicode
-from six import PY3
-
-if PY3:
-    from urllib.parse import (
-        ParseResult,
-        SplitResult,
-        urlparse,
-        unquote,
-        quote,
-        urlunparse,
-    )
-else:
-    from urlparse import ParseResult, SplitResult
-    from urlparse import urlparse, urlunparse
-    from urllib import unquote, quote
-
-
-def uc2utf8(input):
-    # argh!  this feels wrong, but seems to be needed.
-    if not PY3 and type(input) == unicode:
-        return input.encode("utf-8")
-    else:
-        return input
 
 
 class URL:
@@ -32,7 +16,7 @@ class URL:
     This class is for wrapping URLs into objects.  It's used
     internally in the library, end users should not need to know
     anything about this class.  All methods that accept URLs can be
-    fed either with an URL object, a string or an urlparse.ParsedURL
+    fed either with a URL object, a string or a urlparse.ParsedURL
     object.
 
     Addresses may be one out of three:
@@ -86,7 +70,7 @@ class URL:
     def __hash__(self):
         return hash(str(self))
 
-    # TODO: better naming?  Will return url if url is already an URL
+    # TODO: better naming?  Will return url if url is already a URL
     # object, else will instantiate a new URL object
     @classmethod
     def objectify(self, url):
@@ -195,12 +179,12 @@ class URL:
             raise ValueError("%s can't be joined with %s" % (self, path))
 
         if path.path[0] == "/":
-            ret_path = uc2utf8(path.path)
+            ret_path = path.path
         else:
             sep = "/"
             if self.path.endswith("/"):
                 sep = ""
-            ret_path = "%s%s%s" % (self.path, sep, uc2utf8(path.path))
+            ret_path = "%s%s%s" % (self.path, sep, path.path)
         return URL(
             ParseResult(
                 self.scheme or path.scheme,
