@@ -1,5 +1,4 @@
 # encoding: utf-8
-
 import datetime
 
 import caldav
@@ -7,14 +6,18 @@ import caldav
 
 class TestRadicale(object):
 
-    SUMMARIES = set(('Godspeed You! Black Emperor at '
-                     'Cirque Royal / Koninklijk Circus',
-                     'Standard - GBA'))
-    DTSTART = set((datetime.datetime(2011, 3, 4, 20, 0),
-                   datetime.datetime(2011, 1, 15, 20, 0)))
+    SUMMARIES = set(
+        (
+            "Godspeed You! Black Emperor at " "Cirque Royal / Koninklijk Circus",
+            "Standard - GBA",
+        )
+    )
+    DTSTART = set(
+        (datetime.datetime(2011, 3, 4, 20, 0), datetime.datetime(2011, 1, 15, 20, 0))
+    )
 
     def setup(self):
-        URL = 'http://localhost:8080/nicoe/perso/'
+        URL = "http://localhost:8080/nicoe/perso/"
         self.client = caldav.DAVClient(URL)
         self.calendar = caldav.objects.Calendar(self.client, URL)
 
@@ -34,9 +37,8 @@ class TestRadicale(object):
 
 
 class TestTryton(object):
-
     def setup(self):
-        URL = 'http://admin:admin@localhost:9080/caldav/Calendars/Test'
+        URL = "http://admin:admin@localhost:9080/caldav/Calendars/Test"
         self.client = caldav.DAVClient(URL)
         self.calendar = caldav.objects.Calendar(self.client, URL)
 
