@@ -22,24 +22,18 @@ import signal
 import socket
 import sys
 import threading
+from http.server import BaseHTTPRequestHandler
+from http.server import HTTPServer
+from socketserver import ThreadingMixIn
 from time import sleep
 from types import CodeType
 from types import FrameType
+from urllib import parse
+from urllib.parse import urlparse
+from urllib.parse import urlunparse
 
 from caldav.lib.python_utilities import to_local
 from caldav.lib.python_utilities import to_wire
-from six import PY3
-
-if PY3:
-    from urllib import parse
-    from urllib.parse import urlparse, urlunparse
-    from http.server import BaseHTTPRequestHandler, HTTPServer
-    from socketserver import ThreadingMixIn
-else:
-    from urlparse import urlparse as parse
-    from urlparse import urlparse, urlunparse
-    from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
-    from SocketServer import ThreadingMixIn
 
 __version__ = "0.3.1"
 
@@ -284,7 +278,7 @@ def daemonize(logger):
             pass
 
     if os.fork() != 0:
-        # allow the child pid to instanciate the server
+        # allow the child pid to instantiate the server
         # class
         sleep(1)
         sys.exit(0)
