@@ -5,12 +5,13 @@ Rule: None of the tests in this file should initiate any internet
 communication, and there should be no dependencies on a working caldav
 server for the tests in this file.  We use the Mock class when needed
 to emulate server communication.
-
 """
 import pickle
 from datetime import date
 from datetime import datetime
 from datetime import timedelta
+from unittest import mock
+from urllib.parse import urlparse
 
 import caldav
 import icalendar
@@ -24,7 +25,6 @@ from caldav.elements import dav
 from caldav.elements import ical
 from caldav.lib import error
 from caldav.lib import url
-from caldav.lib import vcal
 from caldav.lib.python_utilities import to_normal_str
 from caldav.lib.python_utilities import to_wire
 from caldav.lib.url import URL
@@ -37,17 +37,8 @@ from caldav.objects import FreeBusy
 from caldav.objects import Journal
 from caldav.objects import Principal
 from caldav.objects import Todo
-from six import PY3
 
-
-if PY3:
-    from urllib.parse import urlparse
-    from unittest import mock
-else:
-    from urlparse import urlparse
-    import mock
-
-## Some example icalendar data copied from test_caldav.py
+## Some example icalendar data partly copied from test_caldav.py
 ev1 = """BEGIN:VCALENDAR
 VERSION:2.0
 PRODID:-//Example Corp.//CalDAV Client//EN
@@ -156,7 +147,7 @@ END:VTODO
 END:VCALENDAR"""
 
 
-def MockedDAVResponse(text):
+def MockedDAVResponse(text, davclient=None):
     """
     For unit testing - a mocked DAVResponse with some specific content
     """
@@ -165,7 +156,7 @@ def MockedDAVResponse(text):
     resp.reason = "multistatus"
     resp.headers = {}
     resp.content = text
-    return DAVResponse(resp)
+    return DAVResponse(resp, davclient)
 
 
 def MockedDAVClient(xml_returned):
@@ -213,16 +204,12 @@ class TestExpandRRule:
         data2 = self.yearly.icalendar_instance.subcomponents[1].to_ical()
         assert data1.replace(b"199711", b"199811") == data2
 
-    ## TODO: unskip this one
-    @pytest.mark.skip(
-        "waiting for https://github.com/niccokunzmann/python-recurring-ical-events/issues/97"
-    )
     def testThreeTodo(self):
         self.todo.expand_rrule(start=datetime(1996, 10, 10), end=datetime(1999, 12, 12))
         assert len(self.todo.icalendar_instance.subcomponents) == 3
         data1 = self.todo.icalendar_instance.subcomponents[0].to_ical()
         data2 = self.todo.icalendar_instance.subcomponents[1].to_ical()
-        assert data1.replace(b"199711", b"199811") == data2
+        assert data1.replace(b"19970", b"19980") == data2
 
     def testSplit(self):
         self.yearly.expand_rrule(
@@ -236,6 +223,18 @@ class TestExpandRRule:
             == "19970901T130000Z-123403@example.com"
         )
 
+    def test241(self):
+        """
+        Ref https://github.com/python-caldav/caldav/issues/241
+
+        This seems like sort of a duplicate of testThreeTodo, but the ftests actually started failing
+        """
+        assert len(self.todo.data) > 128
+        self.todo.expand_rrule(
+            start=datetime(1997, 4, 14, 0, 0), end=datetime(2015, 5, 14, 0, 0)
+        )
+        assert len(self.todo.data) > 128
+
 
 class TestCalDAV:
     """
@@ -257,17 +256,26 @@ class TestCalDAV:
         assert response.status == 200
         assert response.tree is None
 
-        if PY3:
-            response = client.put(
-                "/foo/møøh/bar".encode("utf-8"),
-                "bringebærsyltetøy 北京 пиво".encode("utf-8"),
-                {},
-            )
-        else:
-            response = client.put(u"/foo/møøh/bar", "bringebærsyltetøy 北京 пиво", {})  # fmt: skip
+        response = client.put(
+            "/foo/møøh/bar".encode("utf-8"),
+            "bringebærsyltetøy 北京 пиво".encode("utf-8"),
+            {},
+        )
         assert response.status == 200
         assert response.tree is None
 
+    @mock.patch("caldav.davclient.requests.Session.request")
+    def testRequestCustomHeaders(self, mocked):
+        """
+        ref https://github.com/python-caldav/caldav/issues/285
+        """
+        mocked().status_code = 200
+        mocked().headers = {}
+        cal_url = "http://me:hunter2@calendar.møøh.example:80/"
+        client = DAVClient(url=cal_url, headers={"X-NC-CalDAV-Webcal-Caching": "On"})
+        assert client.headers["Content-Type"] == "text/xml"
+        assert client.headers["X-NC-CalDAV-Webcal-Caching"] == "On"
+
     @mock.patch("caldav.davclient.requests.Session.request")
     def testEmptyXMLNoContentLength(self, mocked):
         """
@@ -342,10 +350,12 @@ class TestCalDAV:
             "http://cal.example.com/home/bernard/calendars/"
         )
 
-    def testDateSearch(self):
+    @mock.patch("caldav.CalendarObjectResource.is_loaded")
+    def testDateSearch(self, mocked):
         """
         ## ref https://github.com/python-caldav/caldav/issues/133
         """
+        mocked.__bool__ = lambda self: True
         xml = """<xml><multistatus xmlns="DAV:">
 <response>
     <href>/principals/calendar/home@petroski.example.com/963/43B060B3-A023-48ED-B9E7-6FFD38D5073E.ics</href>
@@ -395,7 +405,9 @@ class TestCalDAV:
         calendar = Calendar(
             client, url="/principals/calendar/home@petroski.example.com/963/"
         )
-        results = calendar.date_search(datetime(2021, 2, 1), datetime(2021, 2, 7))
+        results = calendar.date_search(
+            datetime(2021, 2, 1), datetime(2021, 2, 7), expand=False
+        )
         assert len(results) == 3
 
     def testCalendar(self):
@@ -870,6 +882,91 @@ END:VCALENDAR
             },
         }
 
+    def testHugeTreeParam(self):
+        """
+        With dealing with a huge XML response, such as event containing attachments, XMLParser will throw an exception
+        huge_tree parameters allows to handle this kind of events.
+        """
+
+        xml = """
+<multistatus xmlns="DAV:">
+  <response>
+    <href>/17149682/calendars/testcalendar-84439d0b-ce46-4416-b978-7b4009122c64/</href>
+    <propstat>
+      <prop>
+                </prop>
+      <status>HTTP/1.1 200 OK</status>
+    </propstat>
+    <propstat>
+      <prop>
+        <calendar-data xmlns="urn:ietf:params:xml:ns:caldav"/>
+      </prop>
+      <status>HTTP/1.1 404 Not Found</status>
+    </propstat>
+  </response>
+  <response>
+    <href>/17149682/calendars/testcalendar-84439d0b-ce46-4416-b978-7b4009122c64/20010712T182145Z-123401%40example.com.ics</href>
+    <propstat>
+      <prop>
+        <calendar-data xmlns="urn:ietf:params:xml:ns:caldav">
+
+BEGIN:VCALENDAR
+PRODID:-//MDaemon Technologies Ltd//MDaemon 21.5.2
+VERSION:2.0
+METHOD:PUBLISH
+BEGIN:VEVENT
+UID:
+ 040000008200E0007000B7101A82E0080000000050BF99B19D31
+SEQUENCE:0
+DTSTAMP:20230213T142930Z
+SUMMARY:This a summary of a very bug event
+DESCRIPTION:Description of this very big event
+LOCATION:Somewhere
+ORGANIZER:MAILTO:noreply@test.com
+PRIORITY:5
+ATTACH;VALUE=BINARY;ENCODING=BASE64;FMTTYPE=image/jpeg;
+ X-FILENAME=image001.jpg;X-ORACLE-FILENAME=image001.jpg:
+"""
+        xml += (
+            "gIyIoLTkwKCo2KyIjM4444449QEBAJjBGS0U+Sjk/QD3/2wBDAQsLCw8NDx0QEB09KSMpPT09\n"
+            * 153490
+        )
+        xml += """
+ /Z
+DTSTART;TZID="Europe/Paris":20230310T140000
+DTEND;TZID="Europe/Paris":20230310T150000
+END:VEVENT
+END:VCALENDAR
+</calendar-data>
+      </prop>
+      <status>HTTP/1.1 200 OK</status>
+    </propstat>
+  </response>
+</multistatus>
+"""
+        davclient = MockedDAVClient(xml)
+        resp = mock.MagicMock()
+        resp.headers = {"Content-Type": "text/xml"}
+        resp.content = xml
+
+        ## It seems like the huge_tree flag is not necessary in all
+        ## environments as of 2023-07.  Perhaps versioning issues with
+        ## the lxml library.
+        # davclient.huge_tree = False
+        # try:
+        #    import pdb; pdb.set_trace()
+        #    DAVResponse(resp, davclient=davclient)
+        #    assert False
+        # except Exception as e:
+        #    assert type(e) == lxml.etree.XMLSyntaxError
+
+        davclient.huge_tree = True
+        try:
+            DAVResponse(resp, davclient=davclient)
+            assert True
+        except:
+            assert False
+
     def testFailedQuery(self):
         """
         ref https://github.com/python-caldav/caldav/issues/54
@@ -1150,54 +1247,6 @@ END:VCALENDAR
         if value is not None:
             raise Exception("This should have crashed")
 
-    def test_vcal_fixups(self):
-        """
-        There is an obscure function lib.vcal that attempts to fix up
-        known ical standard breaches from various calendar servers.
-        """
-        broken_ical = [
-            ## This first one contains duplicated DTSTART in the event data
-            """BEGIN:VCALENDAR
-X-EXPANDED:True
-X-MASTER-DTSTART:20200517T060000Z
-X-MASTER-RRULE:FREQ=YEARLY
-BEGIN:VEVENT
-DTSTAMP:20210205T101751Z
-UID:20200516T060000Z-123401@example.com
-DTSTAMP:20200516T060000Z
-SUMMARY:Do the needful
-DTSTART:20210517T060000Z
-DTEND:20210517T230000Z
-RECURRENCE-ID:20210517T060000Z
-END:VEVENT
-BEGIN:VEVENT
-DTSTAMP:20210205T101751Z
-UID:20200516T060000Z-123401@example.com
-DTSTAMP:20200516T060000Z
-SUMMARY:Do the needful
-DTSTART:20220517T060000Z
-DTEND:20220517T230000Z
-RECURRENCE-ID:20220517T060000Z
-END:VEVENT
-BEGIN:VEVENT
-DTSTAMP:20210205T101751Z
-UID:20200516T060000Z-123401@example.com
-DTSTAMP:20200516T060000Z
-SUMMARY:Do the needful
-DTSTART:20230517T060000Z
-DTEND:20230517T230000Z
-RECURRENCE-ID:20230517T060000Z
-END:VEVENT
-END:VCALENDAR"""
-        ]  ## todo: add more broken ical here
-
-        for ical in broken_ical:
-            ## This should raise error
-            with pytest.raises(vobject.base.ValidateError):
-                vobject.readOne(ical).serialize()
-            ## This should not raise error
-            vobject.readOne(vcal.fix(ical)).serialize()
-
     def test_calendar_comp_class_by_data(self):
         calendar = Calendar()
         for (ical, class_) in (
@@ -1223,3 +1272,19 @@ END:VCALENDAR"""
         cal_url = "http://me:hunter2@calendar.example:80/"
         with DAVClient(url=cal_url) as client_ctx_mgr:
             assert isinstance(client_ctx_mgr, DAVClient)
+
+    def testExtractAuth(self):
+        """
+        ref https://github.com/python-caldav/caldav/issues/289
+        """
+        cal_url = "http://me:hunter2@calendar.example:80/"
+        with DAVClient(url=cal_url) as client:
+            assert client.extract_auth_types("Basic\n") == ["basic"]
+            assert client.extract_auth_types("Basic") == ["basic"]
+            assert client.extract_auth_types('Basic Realm=foo;charset="UTF-8"') == [
+                "basic"
+            ]
+            assert client.extract_auth_types("Basic,dIGEST Realm=foo") == [
+                "basic",
+                "digest",
+            ]
